/*	File			NoughtsCrossesCLIENT.cpp
	Purpose			Network Game (Noughts and Crosses) CLIENT
	Author			Richard Anthony	(ar26@gre.ac.uk)
	Date			April 2014 (original December 2003)
*/

#include "stdafx.h"
#include "NoughtsCrossesCLIENT.h"
#include <sys/types.h>
#include <sys/timeb.h>
#pragma warning(disable : 4996)

IMPLEMENT_DYNAMIC(CNoughtsCrossesCLIENT, CDialog)
CNoughtsCrossesCLIENT::CNoughtsCrossesCLIENT(CWnd* pParent /*=NULL*/)
	: CDialog(CNoughtsCrossesCLIENT::IDD, pParent)
{	
}

CNoughtsCrossesCLIENT::~CNoughtsCrossesCLIENT()
{
	CloseConnection();
}

BEGIN_MESSAGE_MAP(CNoughtsCrossesCLIENT, CDialog)
	ON_WM_TIMER()
	ON_BN_CLICKED(IDC_Disconnect, OnBnClickedDisconnect)
	ON_BN_CLICKED(ID_DONE, OnBnClickedTcpAoDone)
	ON_BN_CLICKED(IDC_SET_ALIAS, OnBnClickedSetAlias)
	ON_BN_CLICKED(IDC_SELECT_PLAYER, OnBnClickedSelectPlayer)
	ON_BN_CLICKED(IDC_CELL3, OnBnClickedCell3)
	ON_BN_CLICKED(IDC_CELL4, OnBnClickedCell4)
	ON_BN_CLICKED(IDC_CELL5, OnBnClickedCell5)
	ON_BN_CLICKED(IDC_CELL6, OnBnClickedCell6)
	ON_BN_CLICKED(IDC_CELL7, OnBnClickedCell7)
	ON_BN_CLICKED(IDC_CELL8, OnBnClickedCell8)
	ON_BN_CLICKED(IDC_CELL0, OnBnClickedCell0)
	ON_BN_CLICKED(IDC_CELL1, OnBnClickedCell1)
	ON_BN_CLICKED(IDC_CELL2, OnBnClickedCell2)
END_MESSAGE_MAP()

void CNoughtsCrossesCLIENT::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_ALIAS_NAME, m_AliasName);
	DDX_Control(pDX, IDC_PLAYER_LIST, m_PlayerListBox);
	DDX_Control(pDX, IDC_CELL0, m_Cell0);
	DDX_Control(pDX, IDC_CELL1, m_Cell1);
	DDX_Control(pDX, IDC_CELL2, m_Cell2);
	DDX_Control(pDX, IDC_CELL3, m_Cell3);
	DDX_Control(pDX, IDC_CELL4, m_Cell4);
	DDX_Control(pDX, IDC_CELL5, m_Cell5);
	DDX_Control(pDX, IDC_CELL6, m_Cell6);
	DDX_Control(pDX, IDC_CELL7, m_Cell7);
	DDX_Control(pDX, IDC_CELL8, m_Cell8);
	DDX_Control(pDX, IDC_TOKEN, m_Token);
	DDX_Control(pDX, IDC_STATUS, m_Status);
	DDX_Control(pDX, IDC_OPPONENT_ALIAS, m_OpponentAlias);
	DDX_Control(pDX, IDC_GAME_STATUS, m_GameStatus);
	DDX_Control(pDX, IDC_SELECT_PLAYER, m_SelectPlayer);

	// controls initialisation
	m_pConnectAddress_1 = (CEdit*) GetDlgItem(IDC_ADD_1);
	m_pConnectAddress_2 = (CEdit*) GetDlgItem(IDC_ADD_2);
	m_pConnectAddress_3 = (CEdit*) GetDlgItem(IDC_ADD_3);
	m_pConnectAddress_4 = (CEdit*) GetDlgItem(IDC_ADD_4);
	m_pPort = (CEdit*) GetDlgItem(IDC_PORT);	
	m_pConnectStatus = (CEdit*) GetDlgItem(IDC_ConnectStatus_EDIT);
	m_pServerName = (CEdit*) GetDlgItem(IDC_ServerName_EDIT);

	CString csStr;	
	csStr = "Not connected";
	m_pConnectStatus->SetWindowText(csStr.GetString());

	m_pDisconnectButton = (CButton*) GetDlgItem(IDC_Disconnect);
	m_pRegisterAlias = (CButton*) GetDlgItem(IDC_SET_ALIAS);
	InitialiseControls();
	Initialise_ListeningFor_ServerAdvertisement_broadcasts(); // Set up socket, address structure etc. for listening for Server Advertisements
}

void CNoughtsCrossesCLIENT::InitialiseControls()
{	
	m_pPort->EnableWindow(false);
	m_pConnectAddress_1->EnableWindow(false);
	m_pConnectAddress_2->EnableWindow(false);
	m_pConnectAddress_3->EnableWindow(false);
	m_pConnectAddress_4->EnableWindow(false);
	m_pDisconnectButton->EnableWindow(false);
	m_pRegisterAlias->EnableWindow(false);
	m_AliasName.EnableWindow(false);	
	m_SelectPlayer.EnableWindow(false);
	m_PlayerListBox.EnableWindow(false);
	m_GameStatus.SetWindowText("No active game"); // Display game status
	ClearAllCells();
	m_bSockCreated = false;
	m_bConnected = false;
	m_Token.SetWindowText("");
	m_Status.SetWindowText("");
	m_OpponentAlias.SetWindowText("");		
}

IN_ADDR CNoughtsCrossesCLIENT::GetLocalHostAddressDetails()
{	
	// Get local address as default server address
	// (This simplifies the user's actions because for testing at least, the server may be on the same computer, or at least in same subnet as client)
	char szStr[80];
	DWORD lLen = sizeof(szStr);
    GetComputerNameA(szStr,&lLen);
	hostent* pHost;
	pHost = gethostbyname(szStr);
		
	IN_ADDR addr;
	char*  pChar;
	char** ppChar = pHost->h_addr_list;

	if(ppChar != NULL && *ppChar != NULL)  
	{
		pChar = *ppChar; 
		addr.S_un.S_un_b.s_b1 = (unsigned  char) *pChar++;
		addr.S_un.S_un_b.s_b2 = (unsigned  char) *pChar++;
		addr.S_un.S_un_b.s_b3 = (unsigned  char) *pChar++;
		addr.S_un.S_un_b.s_b4 = (unsigned  char) *pChar;
	}
	else
	{
		MessageBox("Failed to get local address!","Client");
		this->OnCancel();
	}
	return addr;
}

void CNoughtsCrossesCLIENT::Connect_to_Server_using_received_AddressDetails()
{	
	// 1. Create socket
	m_iConnectSocket = socket(AF_INET, SOCK_STREAM, PF_UNSPEC); // Create the socket
	if(INVALID_SOCKET == m_iConnectSocket)
	{
		MessageBox("Socket create failed","CNoughtsCrossesCLIENT");
		return;	
	}

	m_bSockCreated = true;

	// 2. Set blocking mode for socket during connection set-up phase
	u_long lBlocking = 0; // 0 = blocking is enabled, non 0 = non-blocking is enabled.
	int iError = ioctlsocket(m_iConnectSocket, FIONBIO, &lBlocking); // Set blocking socket behaviour
	if(NO_ERROR != iError)
	{
		MessageBox("Could not set Blocking behaviour for socket","CNoughtsCrossesCLIENT");
		return;	
	}

	// 3. Setup Connect Address family
	m_ConnectSockAddr.sin_family = AF_INET;
	
	CString csStr;
	m_pPort->GetWindowText(csStr);
	u_short iPort = (u_short) atol(csStr.GetString());
	m_ConnectSockAddr.sin_port = htons(iPort);
	
	// 4. Connect
	iError = connect(m_iConnectSocket, (const SOCKADDR FAR*)&m_ConnectSockAddr, sizeof(m_ConnectSockAddr));
	if(SOCKET_ERROR == iError)
	{
		MessageBox("Connect failed !\n\nIs server address correct ?\nIs port correct ?\nIs server running ?","CNoughtsCrossesCLIENT");
		closesocket(m_iConnectSocket);		// Close the socket
		return;
	}

	m_bConnected = true;
	csStr = "Connected to Server";
	m_pConnectStatus->SetWindowTextA(csStr);

	// 5. Update controls
	m_pDisconnectButton->EnableWindow(true);
	m_AliasName.EnableWindow(true);	
	m_pRegisterAlias->EnableWindow(true);

	// 6. Set non-blocking (now that connection established) for receiving
	lBlocking = 1; // 0 = blocking is enabled, non 0 = non-blocking is enabled.
	iError = ioctlsocket(m_iConnectSocket, FIONBIO, &lBlocking); // Set non-blocking socket behaviour
	if(NO_ERROR != iError)
	{
		MessageBox("Could not set non-Blocking behaviour for socket","CNoughtsCrossesCLIENT");
		return;	
	}

	// 7. Enable receiving 
	DoReceive();
}

bool CNoughtsCrossesCLIENT::Send(Message_PDU Message)
{
	int iBytesSent;

	iBytesSent = send(m_iConnectSocket, (char *) &Message, sizeof(Message_PDU), 0);
	if(SOCKET_ERROR == iBytesSent)
	{
		int iError = WSAGetLastError();
		StopTimerIfConnectionLost(iError);
		return false;
	}
	return true;
}

void CNoughtsCrossesCLIENT::InitiateTimer()
{	
	m_nTimer = SetTimer(1, 100, 0);	// 100 ms one-shot for UI responsiveness
}

void CNoughtsCrossesCLIENT::Initiate_ListenForserverAdvertisement_Timer()
{	
	m_nTimer2 = SetTimer(2, 1000, 0);	// 1S for responsiveness to server broadcasts
}

void CNoughtsCrossesCLIENT::StopTimer()
{
	KillTimer(m_nTimer);
}

void CNoughtsCrossesCLIENT::OnTimer(UINT nIDEvent) 
{	
	switch (nIDEvent)
	{
	case 1:
        DoReceive();
		break;

	case 2:
		if(m_bConnected == false)
		{
			DoReceive_ServerAdvertisement_broadcast();
		}

		break;
	}
}

void CNoughtsCrossesCLIENT::DoReceive()
{		
	StopTimer();

	int iBytesRecd;
	Message_PDU Message;

	iBytesRecd = recv(m_iConnectSocket, (char *) &Message, sizeof(Message_PDU), 0);
	if(SOCKET_ERROR == iBytesRecd)
	{
		int iError = WSAGetLastError();
		if(WSAEWOULDBLOCK == iError)	// Non-blocking socket used	(if no packet is queued for the socket the operation would otherwise block)

		{
			InitiateTimer();			// 100ms interval
		}
		else
		{
			StopTimerIfConnectionLost(iError);
		}
		return;
	}
	else								// A message was received
	{
		if(iBytesRecd == 0)
		{
			ConnectionLost();
			return;
		}

		switch(Message.iMessageType)
		{
		case PLAYER_LIST:
		  DisplayPlayerList(Message);	
		  break;
		case START_OF_GAME:
			m_SelectPlayer.EnableWindow(false);				// Disable Select Opponent control
			m_OpponentAlias.SetWindowText(Message.cAlias);	// Set and display opponent alias
			if(Message.iToken == 0) 						// Set and display our token
			{
				m_csToken = "0";
				m_csOpponentToken = "X";
			}
			else
			{
				m_csToken = "X";
				m_csOpponentToken = "0";
			}		
			m_Token.SetWindowText(m_csToken.GetString());
			
			if(Message.bFirstMove == true)					// configure first-move logic
			{
				m_Status.SetWindowText("Your Move");
				EnableEmptyCells();
			}
			else
			{
				m_Status.SetWindowText("Opponent's Move");
				DisableEmptyCells();
			}
			m_GameStatus.SetWindowText("Game In Progress"); // Display game status
		  break;
		case END_OF_GAME:
			switch(Message.iCode) 
			{
			case END_OF_GAME_DRAW:
			  MessageBox("The game was a draw","Game Over");	
			  break;
			case END_OF_GAME_WIN:
			  MessageBox("Well done! You won","Game Over");	
			  break;
			case END_OF_GAME_LOOSE:
			  MessageBox("Unlucky - You lost","Game Over");	
			  break;
			case END_OF_GAME_CLIENT_DISCONNECT:
			  MessageBox("Opponent Client Disconnected","Game Over");
			  break;
			}
			
			// Configure controls so that the client can join another game
			m_SelectPlayer.EnableWindow(true);
			m_PlayerListBox.EnableWindow(true);
			m_GameStatus.SetWindowText("No active game"); // Display game status
			ClearAllCells();
			break;
		case OPPONENT_CELL_SELECTION:
		  switch (Message.iCell)
			{
		    case 0:
			  m_Cell0.SetWindowText(m_csOpponentToken.GetString());
			  m_Cell0.EnableWindow(false);
			  break;
		    case 1:
			  m_Cell1.SetWindowText(m_csOpponentToken.GetString());
			  m_Cell1.EnableWindow(false);
			  break;
		    case 2:
			  m_Cell2.SetWindowText(m_csOpponentToken.GetString());
			  m_Cell2.EnableWindow(false);
			  break;
		    case 3:
			  m_Cell3.SetWindowText(m_csOpponentToken.GetString());
			  m_Cell3.EnableWindow(false);
			  break;
		    case 4:
			  m_Cell4.SetWindowText(m_csOpponentToken.GetString());
			  m_Cell4.EnableWindow(false);
			  break;
		    case 5:
			  m_Cell5.SetWindowText(m_csOpponentToken.GetString());
			  m_Cell5.EnableWindow(false);
			  break;
		    case 6:
			  m_Cell6.SetWindowText(m_csOpponentToken.GetString());
			  m_Cell6.EnableWindow(false);
			  break;
		    case 7:
			  m_Cell7.SetWindowText(m_csOpponentToken.GetString());
			  m_Cell7.EnableWindow(false);
			  break;
		    case 8:
			  m_Cell8.SetWindowText(m_csOpponentToken.GetString());
			  m_Cell8.EnableWindow(false);
			  break;
			}
			m_Status.SetWindowText("Your Move");
			EnableEmptyCells();
			break;
		}
	}
	InitiateTimer();
}

void CNoughtsCrossesCLIENT::DisplayPlayerList(Message_PDU Message)
{
	// Clear contents of PlayerList control
	while(m_PlayerListBox.GetCount() > 0)
	{	
		m_PlayerListBox.DeleteString(0);
	}

	CString csAlias;
	m_AliasName.GetWindowText(csAlias); // Read our Alias (exclude it from displayed list)
	
	CString csStr(Message.cPlayerList); // comma-separated, null-terminated list
	int iNextCommaPos;
	CString csCurrentPlayer;
	while((iNextCommaPos = csStr.Find(',')) > 0)
	{
		csCurrentPlayer = csStr.Left(iNextCommaPos);
		if(csCurrentPlayer.CompareNoCase(csAlias) != 0) // Exclude own alias from displayed list
		{
			m_PlayerListBox.AddString(csCurrentPlayer.GetString());
		}
		csStr = csStr.Right(csStr.GetLength() - (iNextCommaPos + 1));
	}
	if(csStr.CompareNoCase(csAlias) != 0) // Exclude own alias from displayed list
	{
		m_PlayerListBox.AddString(csStr.GetString());
	}
	m_PlayerListBox.SetCurSel(0); // Set default selection
}

void CNoughtsCrossesCLIENT::StopTimerIfConnectionLost(int iError)			

{
	if(WSAECONNABORTED == iError || WSAECONNRESET == iError ||
		WSAETIMEDOUT == iError || WSAENETRESET == iError ||
		WSAESHUTDOWN == iError)
	{	
		StopTimer();
		this->OnCancel();
	}
}

void CNoughtsCrossesCLIENT::ConnectionLost()
{
	OnBnClickedTcpAoDone();
}

void CNoughtsCrossesCLIENT::CloseConnection()
{	
	if(m_bConnected)	// if connected - close the connection
	{
		shutdown(m_iConnectSocket,SD_BOTH);	// Close the connection (in both directions)
	}

	if(m_bSockCreated)	// if socket created - close the socket
	{
		closesocket(m_iConnectSocket);	// Close the socket
	}
}

void CNoughtsCrossesCLIENT::OnBnClickedDisconnect()
{
	if(m_bConnected)	// If connected - send disconnection message (graceful disconnect)
	{
		Message_PDU Message;
		Message.iMessageType = CLIENT_DISCONNECT;
		Send(Message);
		CString csStr = "Disconnected";
		m_pConnectStatus->SetWindowTextA(csStr);
	}

	CloseConnection();
	StopTimer();
	InitialiseControls();
}

void CNoughtsCrossesCLIENT::OnBnClickedTcpAoDone()
{
	OnBnClickedDisconnect();
	this->OnCancel();
}

void CNoughtsCrossesCLIENT::OnBnClickedSetAlias() // Register Alias with server
{
	CString csStr;
	m_AliasName.GetWindowText(csStr);
	if(csStr.GetLength() == 0)
	{
		MessageBox("Alias name cannot be blank","'Noughts and Crosses' Client");
	}
	else
	{
		Message_PDU Message;
		Message.iMessageType = REGISTER_ALIAS;
		strncpy(Message.cAlias,csStr.GetString(),MAX_ALIAS_LEN);
		if(Send(Message) == true)
		{
			m_AliasName.EnableWindow(false);	
			m_pRegisterAlias->EnableWindow(false);
			m_PlayerListBox.EnableWindow(true);
			m_SelectPlayer.EnableWindow(true);
		}
	}
}

void CNoughtsCrossesCLIENT::OnBnClickedSelectPlayer()
{
	int iSelection = m_PlayerListBox.GetCurSel();
	if(-1 == iSelection)
	{
		MessageBox("No player selected","'Noughts and Crosses' Client");
	}
	else
	{
		char cBuffer[MAX_ALIAS_LEN + 1];
		int iLen = m_PlayerListBox.GetText(iSelection,cBuffer);
	
		// Send a CHOOSE_OPPONENT message to server
		Message_PDU Message;
		Message.iMessageType = CHOOSE_OPPONENT;
		strcpy(Message.cAlias,cBuffer);
		Send(Message);
	}
}

void CNoughtsCrossesCLIENT::OnBnClickedCell0()
{
	m_Cell0.SetWindowText(m_csToken.GetString());
	m_Cell0.EnableWindow(false);
	CellSelectAction(0);
}

void CNoughtsCrossesCLIENT::OnBnClickedCell1()
{
	m_Cell1.SetWindowText(m_csToken.GetString());
	m_Cell1.EnableWindow(false);
	CellSelectAction(1);
}

void CNoughtsCrossesCLIENT::OnBnClickedCell2()
{
	m_Cell2.SetWindowText(m_csToken.GetString());
	m_Cell2.EnableWindow(false);
	CellSelectAction(2);
}

void CNoughtsCrossesCLIENT::OnBnClickedCell3()
{
	m_Cell3.SetWindowText(m_csToken.GetString());
	m_Cell3.EnableWindow(false);
	CellSelectAction(3);
}

void CNoughtsCrossesCLIENT::OnBnClickedCell4()
{
	m_Cell4.SetWindowText(m_csToken.GetString());
	m_Cell4.EnableWindow(false);
	CellSelectAction(4);
}

void CNoughtsCrossesCLIENT::OnBnClickedCell5()
{
	m_Cell5.SetWindowText(m_csToken.GetString());
	m_Cell5.EnableWindow(false);
	CellSelectAction(5);
}

void CNoughtsCrossesCLIENT::OnBnClickedCell6()
{
	m_Cell6.SetWindowText(m_csToken.GetString());
	m_Cell6.EnableWindow(false);
	CellSelectAction(6);
}

void CNoughtsCrossesCLIENT::OnBnClickedCell7()
{
	m_Cell7.SetWindowText(m_csToken.GetString());
	m_Cell7.EnableWindow(false);
	CellSelectAction(7);
}

void CNoughtsCrossesCLIENT::OnBnClickedCell8()
{
	m_Cell8.SetWindowText(m_csToken.GetString());
	m_Cell8.EnableWindow(false);
	CellSelectAction(8);
}

void CNoughtsCrossesCLIENT::CellSelectAction(int iCell)
{
	// send a CELL_SELECTION message to server
	Message_PDU Message;
	Message.iMessageType = LOCAL_CELL_SELECTION;
	Message.iCell = iCell;
	if(Send(Message) == true)
	{
		m_Status.SetWindowText("Opponent's Move");
		DisableEmptyCells();
	}
}

void CNoughtsCrossesCLIENT::DisableEmptyCells()
{
	CString csStr;
	m_Cell0.GetWindowText(csStr);
	if(csStr.IsEmpty()) m_Cell0.EnableWindow(false); 
	m_Cell1.GetWindowText(csStr);
	if(csStr.IsEmpty()) m_Cell1.EnableWindow(false); 
	m_Cell2.GetWindowText(csStr);
	if(csStr.IsEmpty()) m_Cell2.EnableWindow(false); 
	m_Cell3.GetWindowText(csStr);
	if(csStr.IsEmpty()) m_Cell3.EnableWindow(false); 
	m_Cell4.GetWindowText(csStr);
	if(csStr.IsEmpty()) m_Cell4.EnableWindow(false); 
	m_Cell5.GetWindowText(csStr);
	if(csStr.IsEmpty()) m_Cell5.EnableWindow(false); 
	m_Cell6.GetWindowText(csStr);
	if(csStr.IsEmpty()) m_Cell6.EnableWindow(false); 
	m_Cell7.GetWindowText(csStr);
	if(csStr.IsEmpty()) m_Cell7.EnableWindow(false); 
	m_Cell8.GetWindowText(csStr);
	if(csStr.IsEmpty()) m_Cell8.EnableWindow(false); 
}

void CNoughtsCrossesCLIENT::EnableEmptyCells()
{
	CString csStr;
	m_Cell0.GetWindowText(csStr);
	if(csStr.IsEmpty()) m_Cell0.EnableWindow(true); 
	m_Cell1.GetWindowText(csStr);
	if(csStr.IsEmpty()) m_Cell1.EnableWindow(true); 
	m_Cell2.GetWindowText(csStr);
	if(csStr.IsEmpty()) m_Cell2.EnableWindow(true); 
	m_Cell3.GetWindowText(csStr);
	if(csStr.IsEmpty()) m_Cell3.EnableWindow(true); 
	m_Cell4.GetWindowText(csStr);
	if(csStr.IsEmpty()) m_Cell4.EnableWindow(true); 
	m_Cell5.GetWindowText(csStr);
	if(csStr.IsEmpty()) m_Cell5.EnableWindow(true); 
	m_Cell6.GetWindowText(csStr);
	if(csStr.IsEmpty()) m_Cell6.EnableWindow(true); 
	m_Cell7.GetWindowText(csStr);
	if(csStr.IsEmpty()) m_Cell7.EnableWindow(true); 
	m_Cell8.GetWindowText(csStr);
	if(csStr.IsEmpty()) m_Cell8.EnableWindow(true); 
}

void CNoughtsCrossesCLIENT::ClearAllCells()
{
	m_Cell0.SetWindowText("");
	m_Cell1.SetWindowText("");
	m_Cell2.SetWindowText("");
	m_Cell3.SetWindowText("");
	m_Cell4.SetWindowText("");
	m_Cell5.SetWindowText("");
	m_Cell6.SetWindowText("");
	m_Cell7.SetWindowText("");
	m_Cell8.SetWindowText("");
	DisableEmptyCells();
}

void CNoughtsCrossesCLIENT::InitialiseLocalAddressStructAndPort()
{	// Initialise Local IP address and Port number for listening for Server Advertisement messages
	m_LocalSockAddr.sin_addr = GetLocalHostAddressDetails();
	m_LocalSockAddr.sin_family = AF_INET;
	m_LocalSockAddr.sin_port = htons((u_short) SERVER_ADVERTISE_PORT);
}

void CNoughtsCrossesCLIENT::Initialise_ListeningFor_ServerAdvertisement_broadcasts()
{
	// 1. Create Listen Socket and set it NonBlocking
	m_iListenFor_ServerAdvertisementBroadcasts_Socket = socket(AF_INET, SOCK_DGRAM, PF_UNSPEC); // Create the socket
	if(INVALID_SOCKET == m_iListenFor_ServerAdvertisementBroadcasts_Socket)
	{
		MessageBox("Socket create failed","CNoughtsCrossesCLIENT");
		return;	
	}
	
	// 2. Set non-blocking socket behaviour
	u_long lBlocking = 1; // 0 = blocking is enabled, non 0 = non-blocking is enabled.
	int iError = ioctlsocket(m_iListenFor_ServerAdvertisementBroadcasts_Socket, FIONBIO, &lBlocking); 
	if(NO_ERROR != iError)
	{
		MessageBox("Could not set non-Blocking behaviour for Listen socket","CNoughtsCrossesCLIENT");
		return;	
	}
	
	// 3. Set up local address structure
	InitialiseLocalAddressStructAndPort(); // For listening for Server Advertisement broadcasts

	// 4. Bind   *** NOTE THAT BECAUSE THE CLIENT BINDS TO A SPECIFIC PORT, THIS SOLUTION ONLY PERMITS ONE CLIENT INSTANCE TO RUN AT EACH COMPUTER ***
	int iResult = bind(m_iListenFor_ServerAdvertisementBroadcasts_Socket, (const SOCKADDR FAR*)&m_LocalSockAddr, sizeof(m_LocalSockAddr));
	if(SOCKET_ERROR == iResult)
	{
		MessageBox("Bind failed !\nPort is probably in use","CNoughtsCrossesCLIENT");
		return;
	}

	Initiate_ListenForserverAdvertisement_Timer(); // Enable ServerAdvertisement_Reception (see timer logic)
}

void CNoughtsCrossesCLIENT::DoReceive_ServerAdvertisement_broadcast() // Invoked by timer
{	
	ServerAdvertise_Message_PDU Message;
	int iBytesRecd = recvfrom(m_iListenFor_ServerAdvertisementBroadcasts_Socket, (char FAR*) &Message,
								sizeof(Message), 0, NULL, NULL);
	if(SOCKET_ERROR == iBytesRecd)
	{
	}
	else
	{	
		if(SERVER_ADVERTISE == Message.iMessageType)
		{
			CString csStr = Message.cServerName;
			m_pServerName->SetWindowTextA(csStr);
			
			csStr = "Server discovered";
			m_pConnectStatus->SetWindowTextA(csStr);

			csStr.Format("%d",htons((u_short) Message.iPort));
			m_pPort->SetWindowText(csStr.GetString());

			// Set server's Address
			m_ConnectSockAddr.sin_addr.S_un.S_un_b.s_b1 = (unsigned  char) Message.iIP_Address_Byte_1;
			m_ConnectSockAddr.sin_addr.S_un.S_un_b.s_b2 = (unsigned  char) Message.iIP_Address_Byte_2;
			m_ConnectSockAddr.sin_addr.S_un.S_un_b.s_b3 = (unsigned  char) Message.iIP_Address_Byte_3;
			m_ConnectSockAddr.sin_addr.S_un.S_un_b.s_b4 = (unsigned  char) Message.iIP_Address_Byte_4;

			// Display server's Address
			csStr.Format("%d", m_ConnectSockAddr.sin_addr.S_un.S_un_b.s_b1);
			m_pConnectAddress_1->SetWindowText(csStr);
			csStr.Format("%d", m_ConnectSockAddr.sin_addr.S_un.S_un_b.s_b2);
			m_pConnectAddress_2->SetWindowText(csStr);
			csStr.Format("%d", m_ConnectSockAddr.sin_addr.S_un.S_un_b.s_b3);
			m_pConnectAddress_3->SetWindowText(csStr);
			csStr.Format("%d", m_ConnectSockAddr.sin_addr.S_un.S_un_b.s_b4);
			m_pConnectAddress_4->SetWindowText(csStr);

			Connect_to_Server_using_received_AddressDetails();
		}
	}

}