/*	File			NoughtsCrossesSERVER.cpp
	Purpose			Network Game (Noughts and Crosses) SERVER
	Author			Richard Anthony	(ar26@gre.ac.uk)
	Date			Updated April 2014 (originally December 2003)
*/

#include "stdafx.h"
#include "CaseStudyGame_Server_Main.h"
#include "NoughtsCrossesSERVER.h"
#include <sys/types.h>
#include <sys/timeb.h>
#pragma warning(disable : 4996)

IMPLEMENT_DYNAMIC(CNoughtsCrossesSERVER, CDialog)
CNoughtsCrossesSERVER::CNoughtsCrossesSERVER(CWnd* pParent /*=NULL*/)
	: CDialog(CNoughtsCrossesSERVER::IDD, pParent)
{	
	SeedRandomNumberGenerator();

	for(int i = 0; i < MAX_CONNECTIONS; i++)
	{	
		m_ConnectionArray[i].bInUse = false;
	}	
	for(int i = 0; i < MAX_GAMES; i++)
	{	
		m_GameArray[i].bInUse = false;
	}
}

CNoughtsCrossesSERVER::~CNoughtsCrossesSERVER()
{	
	CloseAll();
}

void CNoughtsCrossesSERVER::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);

	// controls initialisation
	m_pLocalAddress = (CEdit*) GetDlgItem(IDC_ADDRESS);
	m_pListenPort = (CEdit*) GetDlgItem(IDC_PORT);
	m_pEnableServer = (CButton*) GetDlgItem(IDC_ENABLE_SERVER);
	m_pStatus = (CEdit*) GetDlgItem(IDC_STATUS);
	DDX_Control(pDX, IDC_ACTIVE_GAMES_COUNT, m_ActiveGamesCount);
	DDX_Control(pDX, IDC_CONNECTED_COUNT, m_ConnectedCount);

	CString csStr;
	InitialiseLocalAddressStructAndPort();	
	InitialiseControls();
}

BEGIN_MESSAGE_MAP(CNoughtsCrossesSERVER, CDialog)
	ON_WM_TIMER()
	ON_BN_CLICKED(IDC_ENABLE_SERVER, OnBnClickedEnableServer)
	ON_BN_CLICKED(ID_CLOSE, OnBnClickedClose)
	ON_BN_CLICKED(ID_CLEAR_EVENT_LIST, OnBnClickedClearEventList)
END_MESSAGE_MAP()

void CNoughtsCrossesSERVER::InitialiseControls()
{	
	m_pListenPort->EnableWindow(true);
	m_pEnableServer->EnableWindow(true);

	m_bServerEnabled = false;
	CountActiveConnections(); // updates control
	CountActiveGames(); // updates control
}

IN_ADDR CNoughtsCrossesSERVER::GetLocalHostAddressDetails()
{	
	char szStr[80];
	DWORD lLen = sizeof(szStr);
    GetComputerNameA(szStr,&lLen);
	hostent* pHost;
	pHost = gethostbyname(szStr);
		
	IN_ADDR addr;
	char*  pChar;
	char** ppChar = pHost->h_addr_list;

	if(ppChar != NULL && *ppChar != NULL)  
	{
		pChar = *ppChar; 
		addr.S_un.S_un_b.s_b1 = (unsigned  char) *pChar++;
		addr.S_un.S_un_b.s_b2 = (unsigned  char) *pChar++;
		addr.S_un.S_un_b.s_b3 = (unsigned  char) *pChar++;
		addr.S_un.S_un_b.s_b4 = (unsigned  char) *pChar;
	}
	else
	{
		MessageBox("Failed to get local address!","Server");
		this->OnCancel();
	}
	return addr;
}

void CNoughtsCrossesSERVER::InitialiseLocalAddressStructAndPort()
{	// Initialise and display the IP address and Port number
	// The port value can be overwritten by the user, hence the 
	// subsequent use of ReadFinalPortValue() to read the final value
	m_LocalSockAddr.sin_addr = GetLocalHostAddressDetails();
	m_LocalSockAddr.sin_family = AF_INET;
	
	CString csStr;
	csStr.Format("%d",PORT);
	m_pListenPort->SetWindowText(csStr.GetString());
	m_LocalSockAddr.sin_port = htons((u_short) PORT);
	
	// Display the local address value
	csStr.Format("%d.%d.%d.%d", 
		m_LocalSockAddr.sin_addr.S_un.S_un_b.s_b1,
        m_LocalSockAddr.sin_addr.S_un.S_un_b.s_b2, 
		m_LocalSockAddr.sin_addr.S_un.S_un_b.s_b3, 
		m_LocalSockAddr.sin_addr.S_un.S_un_b.s_b4);
	m_pLocalAddress->SetWindowText(csStr.GetString());
}

void CNoughtsCrossesSERVER::ReadFinalPortValue()
{	
	// The Port number can be overwritten by the user since it was
	// automatically set to the default value. Hence the need to read the final value
	CString csStr;
	m_pListenPort->GetWindowText(csStr);
	u_short iPort = (u_short) atol(csStr.GetString());
	m_LocalSockAddr.sin_port = htons(iPort);
}

void CNoughtsCrossesSERVER::Accept()
{
	CString csStr;
	int iIndex = GetIndexOfUnusedConnection();
	if(iIndex == -1)
	{
		csStr = "Accept refused - Maximum number of Connections reached";
		WriteStatusLine(csStr);
		return;
	}
	
	int iRemoteAddrLen = sizeof(SOCKADDR_IN);
	int iError;
	bool bResult = false;
	
	m_ConnectionArray[iIndex].m_iConnectSock = accept(m_iListenSock, (SOCKADDR FAR*)&m_ConnectionArray[iIndex].m_ConnectSockAddr, &iRemoteAddrLen);
	if(INVALID_SOCKET == m_ConnectionArray[iIndex].m_iConnectSock)
	{
		iError = WSAGetLastError();
		// If no connection request is queued for the socket the operation would block.
		if(iError != WSAEWOULDBLOCK)
		{	
			MessageBox("Accept failed","CNoughtsCrossesSERVER");
		}
	}
	else
	{
		// Set the newly-created connection socket to Non-Blocking
		u_long lBlocking = 1; // 0 = blocking is enabled, non 0 = non-blocking is enabled.
		iError = ioctlsocket(m_ConnectionArray[iIndex].m_iConnectSock, FIONBIO, &lBlocking); // Set non-blocking socket behaviour
		if(NO_ERROR != iError)
		{
			MessageBox("Could not set non-Blocking behaviour for connection socket","CNoughtsCrossesSERVER");
			return;	
		}

		m_ConnectionArray[iIndex].bInUse = true;
		csStr.Format("Client %d.%d.%d.%d Connected", 
			m_ConnectionArray[iIndex].m_ConnectSockAddr.sin_addr.S_un.S_un_b.s_b1,
			m_ConnectionArray[iIndex].m_ConnectSockAddr.sin_addr.S_un.S_un_b.s_b2, 
			m_ConnectionArray[iIndex].m_ConnectSockAddr.sin_addr.S_un.S_un_b.s_b3,
			m_ConnectionArray[iIndex].m_ConnectSockAddr.sin_addr.S_un.S_un_b.s_b4);
		WriteStatusLine(csStr);
		CountActiveConnections(); // updates control
	}
}

void CNoughtsCrossesSERVER::OnBnClickedEnableServer()
{
	// 1. Create Listen Socket and set it NonBlocking
	m_iListenSock = socket(AF_INET, SOCK_STREAM, PF_UNSPEC); // Create the socket
	if(INVALID_SOCKET == m_iListenSock)
	{
		MessageBox("Socket create failed","CNoughtsCrossesSERVER");
		return;	
	}
	
	// 2. Set non-blocking socket behaviour
	u_long lBlocking = 1; // 0 = blocking is enabled, non 0 = non-blocking is enabled.
	int iError = ioctlsocket(m_iListenSock, FIONBIO, &lBlocking); 
	if(NO_ERROR != iError)
	{
		MessageBox("Could not set non-Blocking behaviour for Listen socket","CNoughtsCrossesSERVER");
		return;	
	}

	// 3. Read the port value because the user may have changed it from the default
	ReadFinalPortValue();

	// 4. Bind
	int iResult = bind(m_iListenSock, (const SOCKADDR FAR*)&m_LocalSockAddr, sizeof(m_LocalSockAddr));
	if(SOCKET_ERROR == iResult)
	{
		MessageBox("Bind failed !\nPort is probably in use","CNoughtsCrossesSERVER");
		return;
	}
	else
	{
		m_pListenPort->EnableWindow(false);
	}

	// 5. Listen - for TCP connection requests from clients
	iResult = listen(m_iListenSock, SOMAXCONN); 
	if(SOCKET_ERROR == iResult)
	{
		MessageBox("Listen failed","CNoughtsCrossesSERVER");
		return;
	}
	else
	{
		m_bServerEnabled = true;
		m_pEnableServer->EnableWindow(false);
		MainLoop();	
		
		// Only initialise the Server Advertisement after the server has been enabled, and thus the Server's Port number cannot be changed further
		Initialise_ServerAdvertisement();
	}
}

int CNoughtsCrossesSERVER::GetIndexOfUnusedConnection()
{
	for(int i = 0; i < MAX_CONNECTIONS; i++)
	{	
		if(m_ConnectionArray[i].bInUse == false)
		{
			return i;
		}
	}
	return -1;	// Signal that all connections are in use
}

int CNoughtsCrossesSERVER::CountActiveConnections()
{
	int iCount = 0;
	for(int i = 0; i < MAX_CONNECTIONS; i++)
	{	
		if(m_ConnectionArray[i].bInUse == true)
		{
			iCount++;;
		}
	}
	// update number of connections control
	CString csStr;
	csStr.Format("%d",iCount);
	m_ConnectedCount.SetWindowText(csStr.GetString());
	return iCount;
}

int CNoughtsCrossesSERVER::CountActiveGames()
{
	int iCount = 0;
	for(int i = 0; i < MAX_GAMES; i++)
	{	
		if(m_GameArray[i].bInUse == true)
		{
			iCount++;;
		}
	}
	// update number of connections control
	CString csStr;
	csStr.Format("%d",iCount);
	m_ActiveGamesCount.SetWindowText(csStr.GetString());
	return iCount;
}

int CNoughtsCrossesSERVER::FindIndexOfConnectionForAlias(CString csAlias)
{
	for(int i = 0; i < MAX_CONNECTIONS; i++)
	{	
		if(m_ConnectionArray[i].csAlias.CompareNoCase(csAlias) == 0)
		{
			return i;
		}
	}
	return -1;	// Signal that Alias name was not found in connection array
}

int CNoughtsCrossesSERVER::GetIndexOfUnusedGame()
{
	for(int i = 0; i < MAX_GAMES; i++)
	{	
		if(m_GameArray[i].bInUse == false)
		{
			return i;
		}
	}
	return -1;	// Signal that all connections are in use
}

bool CNoughtsCrossesSERVER::IsConnectionPartOfActiveGame(int iConnection)
{
	for(int i = 0; i < MAX_GAMES; i++)
	{	
		if(m_GameArray[i].bInUse == true)
		{
			if(m_GameArray[i].iConnection[0] == iConnection	||
				m_GameArray[i].iConnection[1] == iConnection)
			{	
				return true;
			}
		}
	}
	return false;	// Signal connections is not part of an active game
}

int CNoughtsCrossesSERVER::GetConnectionOpponent(int iConnection)
{
	for(int i = 0; i < MAX_GAMES; i++)
	{	
		if(m_GameArray[i].bInUse == true)
		{
			if(m_GameArray[i].iConnection[0] == iConnection)
			{
				return m_GameArray[i].iConnection[1];
			}	
			if(m_GameArray[i].iConnection[1] == iConnection)
			{
				return m_GameArray[i].iConnection[0];
			}
		}
	}
    return -1;
}

void CNoughtsCrossesSERVER::MainLoop()	// All Accept() and Receive() activity managed here
{
	Accept();
	for(int i = 0; i < MAX_CONNECTIONS; i++)
	{	
		if(m_ConnectionArray[i].bInUse == true)
		{
			DoReceive(i);	// For each valid listen-socket DoReceive()
		}
	}
	InitiateTimer();   // As all activity is non-blocking
}

void CNoughtsCrossesSERVER::InitiateTimer()
{	
	m_nTimer = SetTimer(1, 100, 0);	// Short interval to allow UI responsiveness
}

void CNoughtsCrossesSERVER::Initiate_ServerAdvertisement_Timer()
{	
	m_nTimer_ServerAdvertisement = SetTimer(2, 1000, 0);	// 1 second interval
}


void CNoughtsCrossesSERVER::StopTimer()
{
	KillTimer(m_nTimer);
}

void CNoughtsCrossesSERVER::OnTimer(UINT nIDEvent) 
{	
	switch(nIDEvent) {
		case 1:
			StopTimer();
			MainLoop();
			break;
		case 2:
			Send_ServerAdvertisement_Broadcast();
			break;
	}
}

void CNoughtsCrossesSERVER::Send(int iConnection, Message_PDU Message)
{
	int iError;
	int iBytesSent = send(m_ConnectionArray[iConnection].m_iConnectSock,
					(char *) &Message, sizeof(Message_PDU), 0);
	if(SOCKET_ERROR == iBytesSent)
	{
		iError = WSAGetLastError();
		ConnectionLost(iConnection, iError);
	}
}
				
void CNoughtsCrossesSERVER::SendPlayerList()
{
	Message_PDU Message;
	Message.iMessageType = PLAYER_LIST;
	strcpy(Message.cPlayerList,"");
	
	// List should include all Aliases that are connected, but not involved in active games
	for(int i = 0; i < MAX_CONNECTIONS; i++)
	{	
		if(m_ConnectionArray[i].bInUse == true && IsConnectionPartOfActiveGame(i) == false &&
			m_ConnectionArray[i].csAlias.IsEmpty() == false)
		{
			strcat(Message.cPlayerList,m_ConnectionArray[i].csAlias.GetString());
			strcat(Message.cPlayerList,","); // Needs to be a comma-separated list
		}
	}
	if(strlen(Message.cPlayerList) > 0)
	{
		Message.cPlayerList[strlen(Message.cPlayerList) - 1] = 0; // Null terminate (write over last comma)
	}
	// Send PlayerList to ALL connected players
	for(int i = 0; i < MAX_CONNECTIONS; i++)
	{	
		if(m_ConnectionArray[i].bInUse == true)
		{
			Send(i, Message);
		}
	}
}

void CNoughtsCrossesSERVER::SendOpponentCellSelectionMessage(int iConnection, int iCell)
{
	CString csStr;
	int iOpponentConnection;
	iOpponentConnection = GetConnectionOpponent(iConnection);	// find players opponent
	if(iOpponentConnection < 0)
	{
		csStr.Format("Opponent not found for %d",iConnection);
		WriteStatusLine(csStr);
		return;
	}
	Message_PDU Message;
	Message.iMessageType = OPPONENT_CELL_SELECTION;
	Message.iCell = iCell;
	Send(iOpponentConnection, Message);
}

void CNoughtsCrossesSERVER::DoReceive(int iConnection)
{		
	Message_PDU Message;
	int iBytesRecd = recv(m_ConnectionArray[iConnection].m_iConnectSock, (char *) &Message, sizeof(Message_PDU), 0);
	if(SOCKET_ERROR == iBytesRecd)
	{
		int iError = WSAGetLastError();
		// If no packet is queued for the socket the operation would block.
		if(iError != WSAEWOULDBLOCK)
		{	
			ConnectionLost(iConnection, iError);	
			return;
		}
	}
	else
	{
		if(iBytesRecd == 0)
		{
			return;
		}

		CString csStr;
		switch(Message.iMessageType)
		{
		  case REGISTER_ALIAS:
			csStr.Format("Alias '%s' Registered", Message.cAlias);
			WriteStatusLine(csStr);
			m_ConnectionArray[iConnection].csAlias.Format("%s", Message.cAlias);
			SendPlayerList();
			break;
		  case CHOOSE_OPPONENT:
			if(CreateGame(iConnection, Message) == false)
			{	
				csStr.Format("ChooseOpponent '%s' FAILED", Message.cAlias);
			}
			WriteStatusLine(csStr);
			break;
		  case LOCAL_CELL_SELECTION:
			SendOpponentCellSelectionMessage(iConnection, Message.iCell);
			SetCellInGame(iConnection, Message.iCell);
			if(!CheckForWin(iConnection))
			{
				EndOfGame(iConnection);	// Check for end of game (all cells full)
			}
			break;
		  case CLIENT_DISCONNECT:
			csStr.Format("Client %s Disconnected", m_ConnectionArray[iConnection].csAlias.GetString());
			WriteStatusLine(csStr);
			CloseGameAndConnection(iConnection);
			break;
		}
	}
}

bool CNoughtsCrossesSERVER::CreateGame(int iConnection, Message_PDU Message)
{
	// Check initiating connection is not part of a game
	if(IsConnectionPartOfActiveGame(iConnection) == true)
	{
		return false;
	}
	// Check chosen opponent alias is connected
	CString csAlias(Message.cAlias);
	int iOpponentConnection;
	iOpponentConnection = FindIndexOfConnectionForAlias(csAlias);
	if(iOpponentConnection == -1)
	{
		return false;
	}
	// Check chosen opponent alias connection is not part of a game
	if(IsConnectionPartOfActiveGame(iOpponentConnection) == true)
	{
		return false;
	}
	// Get next available game structure in array
	int iGame;
	iGame = GetIndexOfUnusedGame();
	if(iGame == -1)
	{
		return false;
	}

	m_GameArray[iGame].iConnection[0] = iConnection;
	m_GameArray[iGame].iConnection[1] = iOpponentConnection;
	m_GameArray[iGame].bInUse = true;
	CountActiveGames(); // updates control
	bool bFlag = CreateRandomBool(); // Determine who goes first
	
	// Initialise game Grid
	for(int j = 0; j < 3; j++)
	{	
		for(int k = 0; k < 3; k++)
		{	
			m_GameArray[iGame].iGrid[j][k] = -1; // -1 = cell unoccupied (0 = '0' 1 = 'X')
		}	
	}
			
	// Send updated list of available players
	SendPlayerList();

	// send START_OF_GAME messages to both players
	Message_PDU SendMessage;
	SendMessage.iMessageType = START_OF_GAME;
	strncpy(SendMessage.cAlias,m_ConnectionArray[iOpponentConnection].csAlias.GetString(),MAX_ALIAS_LEN);
	SendMessage.iToken = GetPlayerToken(iGame, iConnection);	
	SendMessage.bFirstMove = bFlag;
	Send(iConnection, SendMessage);
	strncpy(SendMessage.cAlias,m_ConnectionArray[iConnection].csAlias.GetString(),MAX_ALIAS_LEN);
	SendMessage.iToken = GetPlayerToken(iGame, iOpponentConnection);	
	SendMessage.bFirstMove = !bFlag;
	Send(iOpponentConnection, SendMessage);

	CString csStr;
	csStr.Format("Game '%s-%s' created", 
		m_ConnectionArray[m_GameArray[iGame].iConnection[0]].csAlias.GetString(),
		m_ConnectionArray[m_GameArray[iGame].iConnection[1]].csAlias.GetString());
	WriteStatusLine(csStr);
	return true;
}

int CNoughtsCrossesSERVER::FindGameForConnection(int iConnection)			
{
	for(int i = 0; i < MAX_GAMES; i++)
	{	
		if(m_GameArray[i].iConnection[0] == iConnection || 
			m_GameArray[i].iConnection[1] == iConnection)
		{
			return i;
		}
	}
	return -1;
}

int CNoughtsCrossesSERVER::GetPlayerToken(int iGame, int iConnection)			
{
	// Lowest numbered connection is '0'
	if(iGame >= 0 && iGame < MAX_GAMES)
	{
		if(	(m_GameArray[iGame].iConnection[0] == iConnection && 
				m_GameArray[iGame].iConnection[0] < m_GameArray[iGame].iConnection[1])
			||
			(m_GameArray[iGame].iConnection[1] == iConnection && 
				m_GameArray[iGame].iConnection[1] < m_GameArray[iGame].iConnection[0]) )
		{
			return 0;	// player identified by iConnection is '0'
		}
		else
		{
			return 1;	// player identified by iConnection is '1'
		}
	}
	return -1;	// iGame out of range
}

void CNoughtsCrossesSERVER::DoSetCellInGame(int iGame, int iCell, int iToken)			
{
	if((iToken == 0 || iToken == 1) && (iGame >= 0 && iGame < MAX_GAMES) 
		&& (iCell >= 0 && iCell < 9))
	{
		// Cell 0 [0][0], 1 [0][1], 2 [0][2]
		//      3 [1][0], 4 [1][1], 5 [1][2]
		//		6 [2][0], 7 [2][1], 8 [2][2]
		m_GameArray[iGame].iGrid[iCell / 3][iCell % 3] = iToken;
	}
	else
	{
		CString csStr;
		csStr.Format("Could not set cell %d, in game %d", iCell, iGame);
		WriteStatusLine(csStr);
    }
}

void CNoughtsCrossesSERVER::SetCellInGame(int iConnection, int iCell)			
{
	CString csStr;
	int iGame;
	iGame  = FindGameForConnection(iConnection);
	if(iGame  == -1)
	{	
		csStr.Format("Game for connection %d not found", iConnection);
		WriteStatusLine(csStr);
		return;
	}

	// Check if player is '0' or 'X' ('0' is always assigned to the lowest numbered connection)
	int iToken;
	iToken = GetPlayerToken(iGame, iConnection);
	if(iToken  == -1)
	{	
		csStr.Format("Token for connection %d (game %d) not found", iConnection, iGame);
		WriteStatusLine(csStr);
		return;
	}

	DoSetCellInGame(iGame, iCell, iToken);	
}
	
bool CNoughtsCrossesSERVER::CheckForLine(int iGame, int iToken)
{
	// check for win on diagonals
	if(m_GameArray[iGame].iGrid[1][1] == iToken &&
		( (m_GameArray[iGame].iGrid[0][0] == iToken &&
			m_GameArray[iGame].iGrid[2][2] == iToken)
			||
		  (m_GameArray[iGame].iGrid[0][2] == iToken &&
			m_GameArray[iGame].iGrid[2][0] == iToken) ) )
	{
		return true;
	}
	// check for horizontal line win
	for(int j = 0; j < 3; j++)
	{
		if(m_GameArray[iGame].iGrid[j][0] == iToken &&
			m_GameArray[iGame].iGrid[j][1] == iToken &&
			m_GameArray[iGame].iGrid[j][2] == iToken)
		{
			return true;
		}
	}
	// check for vertical line win
	for(int j = 0; j < 3; j++)
	{
		if(m_GameArray[iGame].iGrid[0][j] == iToken &&
			m_GameArray[iGame].iGrid[1][j] == iToken &&
			m_GameArray[iGame].iGrid[2][j] == iToken)
		{
			return true;
		}
	}
	return false;
}

bool CNoughtsCrossesSERVER::CheckForWin(int iConnection)
{
	// Player iConnection has just moved
	// Thus only that player could have won
	int iGame;
	iGame = FindGameForConnection(iConnection);
	int iWinningToken;
	iWinningToken = GetPlayerToken(iGame, iConnection);
	// Seach for a line through the grid
	bool bResult;
	bResult = CheckForLine(iGame, iWinningToken);
	if(bResult == true)
	{
		Message_PDU Message;
		Message.iMessageType = END_OF_GAME;
		Message.iCode = END_OF_GAME_WIN;
		Send(iConnection, Message);
		int iOpponentConnection;
		iOpponentConnection = GetConnectionOpponent(iConnection);
		Message.iCode = END_OF_GAME_LOOSE;
		Send(iOpponentConnection, Message);
		RemoveGameAndConnections(iGame);
	}
	return bResult;
}

bool CNoughtsCrossesSERVER::EndOfGame(int iConnection)
{
	// if no cells free, signal end of game (Draw) to both players
	int iGame;
	iGame = FindGameForConnection(iConnection);
	
	for(int j = 0; j < 3; j++)
	{	
		for(int k = 0; k < 3; k++)
		{	
			if(m_GameArray[iGame].iGrid[j][k] == -1) // -1 = cell unoccupied (0 = '0' 1 = 'X')
			{
				return false;	// at least one unoccupied cell
			}
		}	
	}
	Message_PDU Message;
	Message.iMessageType = END_OF_GAME;
	Message.iCode = END_OF_GAME_DRAW;
	Send(iConnection, Message);
	int iOpponentConnection;
	iOpponentConnection = GetConnectionOpponent(iConnection);
	Send(iOpponentConnection, Message);
	RemoveGameAndConnections(iGame);
	return true;
}

void CNoughtsCrossesSERVER::RemoveGameAndConnections(int iGame)
{
	CloseConnection(m_GameArray[iGame].iConnection[0]);
	m_ConnectionArray[m_GameArray[iGame].iConnection[0]].bInUse = false;
	CloseConnection(m_GameArray[iGame].iConnection[1]);
	m_ConnectionArray[m_GameArray[iGame].iConnection[1]].bInUse = false;
	m_GameArray[iGame].bInUse = false;
	CountActiveGames(); // updates control
}

void CNoughtsCrossesSERVER::ConnectionLost(int iConnection, int iError)			
{
	if(WSAECONNABORTED == iError || WSAECONNRESET == iError ||
		WSAETIMEDOUT == iError || WSAENETRESET == iError ||
		WSAESHUTDOWN == iError)
	{
		CloseGameAndConnection(iConnection);
	}
}

void CNoughtsCrossesSERVER::CloseGameAndConnection(int iConnection)
{	
	int iGame = FindGameForConnection(iConnection);
	if(iGame != -1)
	{
		RemoveGameAndConnections(iGame);
	}
	else
	{
		CloseConnection(iConnection);
	}
}

void CNoughtsCrossesSERVER::CloseConnection(int iConnection)
{	
	shutdown(m_ConnectionArray[iConnection].m_iConnectSock,SD_BOTH);	// Close the connection
	closesocket(m_ConnectionArray[iConnection].m_iConnectSock);			// Close the socket
	m_ConnectionArray[iConnection].bInUse = false;
	m_ConnectionArray[iConnection].csAlias = "";
	CountActiveConnections();
	SendPlayerList();
		
	CString csStr;
	csStr.Format("%d.%d.%d.%d Disconnected", 
		m_ConnectionArray[iConnection].m_ConnectSockAddr.sin_addr.S_un.S_un_b.s_b1,
		m_ConnectionArray[iConnection].m_ConnectSockAddr.sin_addr.S_un.S_un_b.s_b2, 
		m_ConnectionArray[iConnection].m_ConnectSockAddr.sin_addr.S_un.S_un_b.s_b3,
		m_ConnectionArray[iConnection].m_ConnectSockAddr.sin_addr.S_un.S_un_b.s_b4);
	WriteStatusLine(csStr);
}

void CNoughtsCrossesSERVER::CloseAll()
{
	for(int i = 0; i < MAX_CONNECTIONS; i++)	// Close active connections
	{	
		if(m_ConnectionArray[i].bInUse == true)
		{
			CloseConnection(i);
		}
	}

	closesocket(m_iListenSock);		// Close the listen socket
}

void CNoughtsCrossesSERVER::OnBnClickedClose()
{
	CloseAll();
	this->OnCancel();
}

void CNoughtsCrossesSERVER::WriteStatusLine(CString csLine)
{
	CString csExistingStatusText;
	m_pStatus->GetWindowText(csExistingStatusText);
	if(csExistingStatusText.GetLength())	// Create a new line if lines already exist
	{
		csExistingStatusText += "\r\n";
	}
	csExistingStatusText += csLine;
	m_pStatus->SetWindowText(csExistingStatusText.GetString());
}

void CNoughtsCrossesSERVER::SeedRandomNumberGenerator()
{
	__time64_t ltime;
    _time64(&ltime);	// Use the number of Seconds since 1/1/1970 as a seed
	srand((unsigned int)ltime );
}

bool CNoughtsCrossesSERVER::CreateRandomBool()
{
	int iRand = rand()%(1000);
	if(iRand >= 500)
	{
        return true;
	}
	return false;
}

void CNoughtsCrossesSERVER::OnBnClickedClearEventList()
{
	m_pStatus->SetWindowText("");
}

void CNoughtsCrossesSERVER::InitialiseServerAdvertiseAddressStructAndPort()
{	// Initialise the IP address and Port number used for broadcasting the Server Advertisement messages
	m_ServerAdvertiseSockAddr.sin_port = htons((u_short) SERVER_ADVERTISE_PORT);
	m_ServerAdvertiseSockAddr.sin_family = AF_INET;
	m_ServerAdvertiseSockAddr.sin_addr.S_un.S_un_b.s_b1 = 255;
    m_ServerAdvertiseSockAddr.sin_addr.S_un.S_un_b.s_b2 = 255; 
	m_ServerAdvertiseSockAddr.sin_addr.S_un.S_un_b.s_b3 = 255;
	m_ServerAdvertiseSockAddr.sin_addr.S_un.S_un_b.s_b4 = 255;
}

void CNoughtsCrossesSERVER::Initialise_ServerAdvertisement()
{
	// 1. Create Listen Socket and set it NonBlocking
	m_iServerAdvertiseSock = socket(AF_INET, SOCK_DGRAM, PF_UNSPEC); // Create the socket (UDP, hence type = SOCK_DGRAM)
	if(INVALID_SOCKET == m_iServerAdvertiseSock)
	{
		MessageBox("Socket create failed","CNoughtsCrossesSERVER");
		return;	
	}
	
	// Set Broadcast on
	char cOpt[2];
	cOpt[0] = 1; // true
	cOpt[1] = 0; // null terminate the option array
	int iError = setsockopt(m_iServerAdvertiseSock, SOL_SOCKET, SO_BROADCAST, cOpt, sizeof(cOpt)); 
  	if(SOCKET_ERROR == iError)
	{
		MessageBox("Set socket broadcast failed","CNoughtsCrossesSERVER");
		return;	
	}
	
	InitialiseServerAdvertiseAddressStructAndPort();
	Initiate_ServerAdvertisement_Timer();
}

void CNoughtsCrossesSERVER::Send_ServerAdvertisement_Broadcast()
{	
	ServerAdvertise_Message_PDU Message;
	Message.iMessageType = SERVER_ADVERTISE;
	strncpy(Message.cServerName,"Noughts_and_Crosses_SERVER", MAX_SERVER_NAME_LEN);
	Message.iIP_Address_Byte_1 = m_LocalSockAddr.sin_addr.S_un.S_un_b.s_b1;
    Message.iIP_Address_Byte_2 = m_LocalSockAddr.sin_addr.S_un.S_un_b.s_b2;
	Message.iIP_Address_Byte_3 = m_LocalSockAddr.sin_addr.S_un.S_un_b.s_b3;
	Message.iIP_Address_Byte_4 = m_LocalSockAddr.sin_addr.S_un.S_un_b.s_b4;
	Message.iPort = m_LocalSockAddr.sin_port;
			
	// Remember the length of data placed in the send buffer
	int iSendLen = (int) sizeof(Message);

	// Send a UDP datagram containing the user's typed message
	int iBytesSent = sendto(m_iServerAdvertiseSock, (char FAR *) &Message, iSendLen, 0,
			(const struct sockaddr FAR *)&m_ServerAdvertiseSockAddr, sizeof(m_ServerAdvertiseSockAddr));
	if(INVALID_SOCKET == iBytesSent)
	{
		MessageBox("Server Advertise broadcast send failed","CNoughtsCrossesSERVER");
	}
}