/*	File			NoughtsCrossesSERVER.h
	Purpose			Network Game (Noughts and Crosses) SERVER
	Author			Richard Anthony	(ar26@gre.ac.uk)
	Date			Updated April 2014 (originally December 2003)
*/
#pragma once
#include "NoughtsCrosses_General.h"
#include "afxwin.h"

#define MAX_GAMES 5
#define MAX_CONNECTIONS MAX_GAMES * 2

struct Connection {
	SOCKET m_iConnectSock;
	SOCKADDR_IN m_ConnectSockAddr;
	CString csAlias;
	bool bInUse;
};

struct Game {
	int iConnection[2];
	bool bInUse;
	int iGrid[3][3];
};

class CNoughtsCrossesSERVER : public CDialog
{
	DECLARE_DYNAMIC(CNoughtsCrossesSERVER)

public:
	CNoughtsCrossesSERVER(CWnd* pParent = NULL);   // standard constructor
	virtual ~CNoughtsCrossesSERVER();
	void InitialiseControls();
	IN_ADDR GetLocalHostAddressDetails();
	void InitialiseLocalAddressStructAndPort();
	void ReadFinalPortValue();
	void InitiateTimer();					// REPEAT for non-blocking recv's
	void StopTimer();
	void OnTimer(UINT nIDEvent); 
	void DoReceive(int iConnection);
	void CloseAll();
	void WriteStatusLine(CString csLine);
	int GetIndexOfUnusedConnection();
	int GetIndexOfUnusedGame();
	int CountActiveConnections();
	int CountActiveGames();
	void ConnectionLost(int iConnection, int iError);			
	void CloseConnection(int iConnection);
	void SendPlayerList();
	bool CreateGame(int iConnection, Message_PDU Message);
	bool IsConnectionPartOfActiveGame(int iConnection);
	int FindIndexOfConnectionForAlias(CString csAlias);
	void SendOpponentCellSelectionMessage(int iConnection, int iCell);
	int GetConnectionOpponent(int iConnection);
	int FindGameForConnection(int iConnection);	
	int GetPlayerToken(int iGame, int iConnection);		
	void DoSetCellInGame(int iGame, int iCell, int iToken);	
	void SetCellInGame(int iConnection, int iCell);
	bool CheckForWin(int iConnection);
	bool EndOfGame(int iConnection);
	bool CheckForLine(int iGame, int iToken);
	void RemoveGameAndConnections(int iGame);
	void SeedRandomNumberGenerator();
	bool CreateRandomBool();
	void CloseGameAndConnection(int iConnection);
	void Accept();
	void Send(int iConnection, Message_PDU Message);
	void MainLoop();
	
	// Methods added to implement Server Advertisement
	void InitialiseServerAdvertiseAddressStructAndPort();
	void Initialise_ServerAdvertisement();
	void Initiate_ServerAdvertisement_Timer();
	void Send_ServerAdvertisement_Broadcast();

	enum { IDD = IDD_NoughtsCrossesSERVER_DIALOG };

protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support

	DECLARE_MESSAGE_MAP()
private:
	CEdit* m_pListenPort;
	CEdit* m_pLocalAddress;
	CEdit* m_pStatus;
	CEdit m_ActiveGamesCount;
	CEdit m_ConnectedCount;
	CButton* m_pEnableServer;
	
	SOCKET m_iListenSock;
	SOCKET m_iServerAdvertiseSock;			// The socket used for sending Server Advertisement broadcasts

	SOCKADDR_IN m_LocalSockAddr;
	SOCKADDR_IN m_ServerAdvertiseSockAddr;	// The address structure used for sending Server Advertisement broadcasts

	Connection m_ConnectionArray[MAX_CONNECTIONS];
	Game m_GameArray[MAX_GAMES];

	// Flags
	bool m_bServerEnabled;

	// Timer
	UINT_PTR m_nTimer;
	UINT_PTR m_nTimer_ServerAdvertisement;

public:
	afx_msg void OnBnClickedEnableServer();
	afx_msg void OnBnClickedClose();
	afx_msg void OnBnClickedClearEventList();
};

